/*
    This file is a part of the SteamWar software.

    Copyright (C) 2021  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.steamwar.fightsystem.utils;

import de.steamwar.fightsystem.Config;
import de.steamwar.fightsystem.fight.FightTeam;
import de.steamwar.fightsystem.listener.Recording;
import de.steamwar.fightsystem.record.GlobalRecorder;
import net.minecraft.server.v1_8_R3.DataWatcher;
import net.minecraft.server.v1_8_R3.EntityEnderDragon;
import net.minecraft.server.v1_8_R3.PacketPlayOutEntityMetadata;
import net.minecraft.server.v1_8_R3.PacketPlayOutSpawnEntityLiving;
import org.bukkit.Effect;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.craftbukkit.v1_8_R3.entity.CraftPlayer;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.bukkit.scoreboard.Team;

import java.util.HashSet;
import java.util.Set;

public class BountifulWrapper8 implements BountifulWrapper {

    public BountifulWrapper8() {
        EntityEnderDragon dragon = new EntityEnderDragon(null);
        dragon.setLocation(Config.ArenaRegion.centerX(), -100, Config.ArenaRegion.centerZ(), 0, 0);
        this.spawnDragonId = dragon.getId();
        this.spawnDragon = new PacketPlayOutSpawnEntityLiving(dragon);
    }

    @Override
    public boolean mainHand(Object packet) {
        return true;
    }

    @Override
    public boolean bowInHand(boolean mainHand, Player p) {
        return p.getInventory().getItemInHand().getType() == Material.BOW;
    }

    @Override
    public void setAttackSpeed(Player player) {
        // nothing to do
    }

    @Override
    public void setNametagVisibility(Team team) {
        //nothing to do
    }

    @Override
    public Listener newDenyArrowPickupListener() {
        return new Listener() {};
    }

    @Override
    public Listener newDenyHandSwapListener() {
        return new Listener() {};
    }

    @Override
    public void recordHandItems(Player player) {
        GlobalRecorder.getInstance().item(player, Recording.disarmNull(player.getInventory().getItemInHand()), "MAINHAND");
    }

    @Override
    public Listener newHandSwapRecorder() {
        return new Listener() {};
    }

    @Override
    public void spawnParticle(World world, String particleName, double x, double y, double z) {
        world.playEffect(new Location(world, x, y, z), Effect.valueOf(particleName), 1);
    }

    private final Set<Player> seesDragon = new HashSet<>();
    private final PacketPlayOutSpawnEntityLiving spawnDragon;
    private final int spawnDragonId;
    @Override
    public void sendBar(Player player, FightTeam team, double progress, String text) {
        seesDragon.removeIf(p -> !p.isOnline());

        if(!seesDragon.contains(player)) {
            ((CraftPlayer)player).getHandle().playerConnection.sendPacket(spawnDragon);
            seesDragon.add(player);
        }

        DataWatcher watcher = new DataWatcher(null);
        watcher.a(0, (byte) 0x20);
        watcher.a(2, text);
        watcher.a(3, (byte) 1);
        watcher.a(4, (byte) 1);
        watcher.a(6, (float)(progress * 200));
        ((CraftPlayer) player).getHandle().playerConnection.sendPacket(new PacketPlayOutEntityMetadata(spawnDragonId, watcher, true));
    }
}
