/*
 *   This file is a part of the SteamWar software.
 *
 *   Copyright (C) 2023  SteamWar.de-Serverteam
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Affero General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Affero General Public License for more details.
 *
 *   You should have received a copy of the GNU Affero General Public License
 *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.steamwar.fightsystem.ai;

import de.steamwar.fightsystem.Config;
import de.steamwar.fightsystem.fight.FightTeam;
import de.steamwar.sql.SchematicNode;
import de.steamwar.sql.SteamwarUser;
import org.bukkit.util.Vector;

import java.util.List;
import java.util.Random;

public class LixfelAI extends AI {

    private final Random random = new Random();
    private LixfelPathplanner pathplanner;

    public LixfelAI(FightTeam team, String user) {
        super(team, SteamwarUser.get(user));
    }


    @Override
    public SchematicNode chooseSchematic() {
        List<SchematicNode> publics = SchematicNode.getAllSchematicsOfType(0, Config.SchematicType.toDB());
        SchematicNode schem = publics.get(new Random().nextInt(publics.size()));
        pathplanner = new LixfelPathplanner(schem);
        return schem;
    }

    @Override
    protected void plan() {
        setReady();
        Vector destination = pathplanner.getWalkable().get(random.nextInt(pathplanner.getWalkable().size()));
        List<Vector> path = pathplanner.plan(getPosition(), destination);
        if(!path.isEmpty())
            chat("Path size: " + path.size());
        for(Vector p : path) {
            move(p);
        }
    }
}
