/* 
    This file is a part of the SteamWar software.
    
    Copyright (C) 2020  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package de.steamwar.fightsystem.commands;

import de.steamwar.fightsystem.ArenaMode;
import de.steamwar.fightsystem.FightSystem;
import de.steamwar.fightsystem.fight.Fight;
import de.steamwar.fightsystem.fight.FightPlayer;
import de.steamwar.fightsystem.fight.FightTeam;
import de.steamwar.fightsystem.fight.JoinRequest;
import de.steamwar.fightsystem.states.FightState;
import de.steamwar.fightsystem.states.StateDependentCommand;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import java.util.function.BiConsumer;

public class RequestsCommand implements CommandExecutor {

    public RequestsCommand() {
        new StateDependentCommand(ArenaMode.VariableTeams, FightState.AntiSpectate, "request", this);
        new StateDependentCommand(ArenaMode.VariableTeams, FightState.AntiSpectate, "requests", this);
    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        if(!(sender instanceof Player))
            return false;
        Player player = (Player) sender;
        FightPlayer fp = Fight.getFightPlayer(player);
        if(fp == null || !(fp.isLeader() || fp.isLiving())) {
            GUI.joinRequest(player);
            return false;
        }

        if(Commands.checkGetLeader(player) == null)
            return false;

        GUI.chooseJoinRequests(player);
        return false;
    }

    public static void onJoinRequest(Player player, Player target, BiConsumer<JoinRequest, FightTeam> handleJoinRequest) {
        JoinRequest request = JoinRequest.get(target);
        if(request == null) {
            FightSystem.getMessage().send("NO_JOIN_REQUEST", player);
            return;
        }

        FightTeam team = Fight.getPlayerTeam(player);
        if(!request.required(team)) {
            FightSystem.getMessage().send("NO_CONFIRMATION", player);
            return;
        }

        handleJoinRequest.accept(request, team);
    }
}
