/* 
    This file is a part of the SteamWar software.
    
    Copyright (C) 2020  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package de.steamwar.fightsystem.listener;

import de.steamwar.fightsystem.ArenaMode;
import de.steamwar.fightsystem.Config;
import de.steamwar.fightsystem.FightSystem;
import de.steamwar.fightsystem.fight.Fight;
import de.steamwar.fightsystem.fight.FightTeam;
import de.steamwar.fightsystem.record.GlobalRecorder;
import de.steamwar.fightsystem.states.FightState;
import de.steamwar.fightsystem.states.StateDependentListener;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.AsyncPlayerChatEvent;

import java.util.logging.Level;

public class Chat implements Listener {

    public Chat(){
        new StateDependentListener(ArenaMode.All, FightState.All, this);
    }

    @EventHandler
    public void handlePlayerChat(AsyncPlayerChatEvent event) {
        Player player = event.getPlayer();
        String message = event.getMessage();

        FightSystem.getPlugin().getLogger().log(Level.INFO, player.getName() + "» " + message);
        FightTeam fightTeam = Fight.getPlayerTeam(player);
        if(fightTeam != null) {
            String teamName = fightTeam.getColoredName();
            if(message.startsWith(Config.TeamChatDetection)) {
                fightTeam.broadcastChat(player, message.substring(1));
            } else {
                broadcastChat("PARTICIPANT_CHAT", teamName, player.getName(), message);
            }
        }else if(Config.isReferee(player)){
            broadcastChat("FIGHTLEADER_CHAT", player.getName(), message);
        }else{
            broadcastChat("SPECTATOR_CHAT", player.getName(), message);
        }

        event.setCancelled(true);
    }

    public static void broadcastChat(String message, Object... params) {
        GlobalRecorder.getInstance().chat(message, params);
        FightSystem.getMessage().chat(message, params);
    }
}
