/* 
    This file is a part of the SteamWar software.
    
    Copyright (C) 2020  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package de.steamwar.fightsystem.listener;

import de.steamwar.fightsystem.ArenaMode;
import de.steamwar.fightsystem.Config;
import de.steamwar.fightsystem.FightSystem;
import de.steamwar.fightsystem.states.FightState;
import de.steamwar.fightsystem.states.StateDependentListener;
import de.steamwar.sql.SchematicNode;
import de.steamwar.sql.SteamwarUser;
import de.steamwar.sql.UserPerm;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerCommandPreprocessEvent;
import org.bukkit.event.player.PlayerJoinEvent;

import java.util.logging.Level;

public class Check implements Listener {

    public Check() {
        new StateDependentListener(ArenaMode.Check, FightState.All, this);
    }

    @EventHandler
    public void onJoin(PlayerJoinEvent e){
        Player player = e.getPlayer();
        SteamwarUser user = SteamwarUser.get(player.getUniqueId());

        if(user.hasPerm(UserPerm.CHECK))
            return;

        SchematicNode schem = SchematicNode.getSchematicNode(Config.CheckSchemID);
        if(user.getId() == schem.getOwner())
            return;

        FightSystem.getMessage().send("CHECK_JOIN_DENIED", player);
        player.kickPlayer("");
    }

    @EventHandler
    public void handlePlayerCommandPreprocess(PlayerCommandPreprocessEvent event) {
        Player player = event.getPlayer();
        if(!event.getMessage().contains("copy") && !event.getMessage().contains("cut"))
            return;

        event.setCancelled(true);
        FightSystem.getMessage().send("CHECK_COMMAND_LOCKED", player);
        Bukkit.getLogger().log(Level.SEVERE, player.getName() + " tried to use a copy command!");
    }
}
