/* 
    This file is a part of the SteamWar software.
    
    Copyright (C) 2020  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package de.steamwar.fightsystem.listener;

import de.steamwar.fightsystem.ArenaMode;
import de.steamwar.fightsystem.fight.Fight;
import de.steamwar.fightsystem.fight.FightPlayer;
import de.steamwar.fightsystem.states.FightState;
import de.steamwar.fightsystem.states.StateDependentListener;
import de.steamwar.fightsystem.utils.FightUI;
import de.steamwar.fightsystem.utils.SWSound;
import org.bukkit.entity.Entity;
import org.bukkit.entity.LivingEntity;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.EntityDeathEvent;
import org.bukkit.event.player.PlayerQuitEvent;

import java.util.function.Consumer;

public class IngameDeath implements Listener {

    public IngameDeath() {
        new StateDependentListener(ArenaMode.AntiReplay, FightState.Ingame, this);
    }

    @EventHandler
    public void broadcastDeath(EntityDeathEvent event) {
        onPlayerEnd(event.getEntity(), fightPlayer -> {
            FightUI.addSubtitle("UI_PLAYER_DEATH", fightPlayer.getTeam().getPrefix(), fightPlayer.getEntity().getName());
            Fight.playSound(SWSound.ENTITY_WITHER_DEATH.getSound(), 100.0F, 1.0F);
        });
    }

    @EventHandler
    public void broadcastQuit(PlayerQuitEvent event) {
        onPlayerEnd(event.getPlayer(), fightPlayer -> FightUI.addSubtitle("UI_PLAYER_LEAVE", fightPlayer.getTeam().getPrefix(), fightPlayer.getEntity().getName()));
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void handlePlayerDeath(EntityDeathEvent event) {
        onPlayerEnd(event.getEntity(), FightPlayer::setOut);
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void handlePlayerQuit(PlayerQuitEvent event) {
        onPlayerEnd(event.getPlayer(), FightPlayer::setOut);
    }

    private void onPlayerEnd(Entity player, Consumer<FightPlayer> withLivingPlayer) {
        if(!(player instanceof LivingEntity))
            return;

        FightPlayer fightPlayer = Fight.getFightPlayer((LivingEntity) player);
        if(fightPlayer == null || !fightPlayer.isLiving())
            return;

        withLivingPlayer.accept(fightPlayer);
    }
}
