/*
    This file is a part of the SteamWar software.

    Copyright (C) 2022  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.steamwar.fightsystem.listener;

import de.steamwar.fightsystem.Config;
import de.steamwar.fightsystem.fight.Fight;
import de.steamwar.fightsystem.states.FightState;
import de.steamwar.fightsystem.states.StateDependentListener;
import org.bukkit.GameMode;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerMoveEvent;
import org.bukkit.event.player.PlayerRespawnEvent;
import org.spigotmc.event.player.PlayerSpawnLocationEvent;

import java.util.HashMap;
import java.util.Map;

public class LeaveableArena implements Listener {

    private final Map<Player, GameMode> spectatorsInArena = new HashMap<>();

    public LeaveableArena() {
        new StateDependentListener(Config.ArenaLeaveable, FightState.All, this);
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onJoin(PlayerSpawnLocationEvent event) {
        Player player = event.getPlayer();
        if(Fight.fighting(player))
            return;

        if(Config.ArenaRegion.in2dRegion(event.getSpawnLocation())) {
            markInArena(player);
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onRespawn(PlayerRespawnEvent event) {
        Player player = event.getPlayer();
        if(Config.ArenaRegion.in2dRegion(event.getRespawnLocation())) {
            markInArena(player);
        } else {
            spectatorsInArena.remove(player);
        }
    }

    @EventHandler
    public void arenaBorder(PlayerMoveEvent event){
        Player player = event.getPlayer();
        if(Fight.fighting(player))
            return;

        boolean inArena = Config.ArenaRegion.in2dRegion(event.getTo());
        boolean spectator = spectatorsInArena.containsKey(player);

        if(inArena && !spectator) {
            markInArena(player);
        } else if(!inArena && spectator) {
            GameMode mode = spectatorsInArena.remove(player);
            if(mode != null)
                player.setGameMode(mode);
        }
    }

    private void markInArena(Player player) {
        spectatorsInArena.put(player, player.getGameMode());
        Fight.setPlayerGamemode(player, GameMode.SPECTATOR);
    }
}
