/*
    This file is a part of the SteamWar software.

    Copyright (C) 2020  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package de.steamwar.fightsystem.record;

import de.steamwar.fightsystem.Config;
import de.steamwar.fightsystem.states.FightState;
import de.steamwar.fightsystem.states.StateDependent;
import org.bukkit.Bukkit;

import java.io.*;
import java.net.Socket;
import java.util.logging.Level;

public class LiveRecorder extends StateDependent implements Recorder {

    private DataOutputStream outputStream;
    private Socket socket;

    public LiveRecorder(){
        super(Config.LiveReplay, FightState.AntiSpectate);
        register();
    }

    @Override
    public void enable() {
        try {
            socket = new Socket(Config.spectateIP, Config.SpectatePort);
            socket.setSoTimeout(1); // Wait a maximum of 1ms on a blocking operation (flush)
            socket.setSoLinger(true, 1); // Wait a maximum of 1ms on disable
            socket.setTcpNoDelay(true); // Don't wait on ack
            outputStream = new DataOutputStream(new BufferedOutputStream(socket.getOutputStream(), 1024));
        } catch (IOException e) {
            Bukkit.getLogger().log(Level.WARNING, "Could not init connection to spectate server", e);
            disable();
        }
        Recorder.super.enable();
    }

    @Override
    public void disable() {
        try {
            socket.close();
        } catch (IOException e) {
            Bukkit.getLogger().log(Level.WARNING, "IOException on socket close", e);
        }
        Recorder.super.disable();
    }

    @Override
    public DataOutputStream getStream() {
        return outputStream;
    }
}
