/*
    This file is a part of the SteamWar software.

    Copyright (C) 2020  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package de.steamwar.fightsystem.states;

import de.steamwar.fightsystem.ArenaMode;
import de.steamwar.fightsystem.Config;
import de.steamwar.fightsystem.winconditions.Winconditions;

import java.util.Set;

public abstract class StateDependent implements IStateDependent {

    private final Set<FightState> enabled;
    private final boolean register;

    protected StateDependent(Winconditions wincondition, Set<FightState> states){
        this(Config.ActiveWinconditions.contains(wincondition), states);
    }

    protected StateDependent(Set<ArenaMode> mode, Set<FightState> states){
        this(mode.contains(Config.mode), states);
    }

    protected StateDependent(boolean enabled, Set<FightState> states){
        this.enabled = states;
        this.register = enabled;
    }

    public void register(){
        if(register)
            FightState.registerStateDependent(this);
    }

    @Override
    public Set<FightState> enabled() {
        return enabled;
    }
}
