/*
    This file is a part of the SteamWar software.

    Copyright (C) 2020  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package de.steamwar.fightsystem.states;

import de.steamwar.fightsystem.ArenaMode;
import de.steamwar.fightsystem.Config;
import de.steamwar.fightsystem.countdown.Countdown;
import de.steamwar.fightsystem.winconditions.Winconditions;

import java.util.Set;

public class StateDependentCountdown extends StateDependent {

    private static Countdown mainCountdown = null;

    public static Countdown getMainCountdown() {
        return mainCountdown;
    }

    private final Countdown countdown;

    public StateDependentCountdown(Set<ArenaMode> mode, Set<FightState> states, Countdown countdown) {
        super(mode, states);
        this.countdown = countdown;
        register();
    }

    public StateDependentCountdown(Winconditions wincondition, Set<FightState> states, Countdown countdown) {
        this(Config.ActiveWinconditions.contains(wincondition), states, countdown);
    }

    public StateDependentCountdown(boolean active, Set<FightState> states, Countdown countdown) {
        super(active, states);
        this.countdown = countdown;
        register();
    }

    public Countdown getCountdown(){
        return countdown;
    }

    @Override
    public void enable() {
        mainCountdown = countdown;
        countdown.enable();
    }

    @Override
    public void disable() {
        countdown.disable();
        if(mainCountdown == countdown)
            mainCountdown = null;
    }
}
