/*
    This file is a part of the SteamWar software.

    Copyright (C) 2020  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package de.steamwar.fightsystem.states;

import de.steamwar.fightsystem.ArenaMode;
import de.steamwar.fightsystem.Config;
import de.steamwar.fightsystem.FightSystem;
import de.steamwar.fightsystem.winconditions.Winconditions;
import org.bukkit.Bukkit;
import org.bukkit.scheduler.BukkitTask;

import java.util.Set;

public class StateDependentTask extends StateDependent {

    private final Runnable runnable;
    private final long delay;
    private final long period;

    private BukkitTask task = null;

    public StateDependentTask(Winconditions wincondition, Set<FightState> states, Runnable runnable, long delay, long period){
        this(Config.ActiveWinconditions.contains(wincondition), states, runnable, delay, period);
    }

    public StateDependentTask(boolean enabled, Set<FightState> states, Runnable runnable, long delay, long period){
        super(enabled, states);
        this.runnable = runnable;
        this.delay = delay;
        this.period = period;
        register();
    }

    public StateDependentTask(Set<ArenaMode> mode, Set<FightState> states, Runnable runnable, long delay, long period) {
        super(mode, states);
        this.runnable = runnable;
        this.delay = delay;
        this.period = period;
        register();
    }

    @Override
    public void enable() {
        task = Bukkit.getScheduler().runTaskTimer(FightSystem.getPlugin(), runnable, delay, period);
    }

    @Override
    public void disable() {
        task.cancel();
    }

}
