/* 
    This file is a part of the SteamWar software.
    
    Copyright (C) 2020  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package de.steamwar.fightsystem.utils;

import de.steamwar.fightsystem.fight.Fight;
import de.steamwar.fightsystem.fight.FightPlayer;
import de.steamwar.fightsystem.fight.FightTeam;
import de.steamwar.fightsystem.states.FightState;
import de.steamwar.fightsystem.states.IStateDependent;

import java.util.Set;

public class EnterHandler implements IStateDependent {
    private static final Set<FightState> enabled = FightState.Running;

    public EnterHandler(){
        FightState.registerStateDependent(this);
    }

    @Override
    public Set<FightState> enabled() {
        return enabled;
    }

    @Override
    public void enable() {
        Fight.teams().forEach(this::registerTeam);
    }

    @Override
    public void disable() {
        Fight.teams().forEach(this::unregisterTeam);
    }

    private void registerTeam(FightTeam team){
        for(FightPlayer fp : team.getPlayers()){
            fp.startEnternCountdown(null);
        }
    }

    private void unregisterTeam(FightTeam team){
        for(FightPlayer fp : team.getPlayers()){
            fp.stopEnternCountdown();
        }
    }
}
